import unittest
# from solution_ex import *
from objects_ex import *

class TestEx1(unittest.TestCase):
    """ tests product class"""

    def test_creation(self):
        p = Product("radio", 10.0, 5)
        self.assertEqual(p.name, "radio")
        self.assertEqual(p.price, 10.0)
        self.assertEqual(p.tax, 5)

    def test_gross(self):
        p = Product("radio", 10.0, 5)
        self.assertEqual(p.gross(), round(10.0 * 1.05, 2))

    def test_net(self):
        p = Product("radio", 10.0, 5)
        self.assertEqual(p.net(), 10.00)

    def test_str(self):
        p = Product("radio", 10.0, 5)
        self.assertEqual(str(p), f"radio: {p.gross()}")

    def test_repr(self):
        p = Product("radio", 10.0, 5)
        self.assertEqual(repr(p), "Product('radio', 10.0, 5)")
        

class TestEx2(unittest.TestCase):
    """ tests basketclass"""

    def test_add_item_to_basket(self):
        p = Product("radio", 10.0, 5)
        b = Basket()
        b.add_product(p)
        b.add_product(p)
        self.assertEqual(b.total_net(), 20.0)
        
        
    def test_gross_on_basket(self):
        p = Product("radio", 10.0, 0)
        b = Basket()
        b.add_product(p)
        b.add_product(p)
        self.assertEqual(b.total_gross(), 20.0)
        
        
    def test_clear_basket(self):
        p = Product("radio", 10.0, 0)
        b = Basket()
        b.add_product(p)
        b.add_product(p)
        b.clear_basket()
        self.assertEqual(b.total_gross(), 0.0)
        self.assertEqual(b.total_net(), 0.0)
    
    def test_remove_product(self):
        radio = Product("radio", 10.0, 0)
        tv = Product("tv", 100.0, 10)
        
        b = Basket()
        b.add_product(radio)
        b.add_product(tv)
        
        self.assertEqual(b.total_gross(), 120.0)
        b.remove_product(tv)
        
        self.assertEqual(b.total_gross(), 10.0)
        
class TestEx3(unittest.TestCase):
    """ tests discount """

    def test_discount_not_enough_items(self):
        p = Product("radio", 10.0, 5)
        b = Basket()
        b.add_product(p)
        b.add_product(p)

        b.add_discount(10, 3)
        self.assertEqual(b.total_net(), 20.0)

    def test_discount(self):
        p = Product("radio", 10.0, 10)
        b = Basket()
        b.add_discount(50, 2)
        b.add_product(p)
        self.assertEqual(b.total_net(), 10.0)

        # 2 products mean 50% discount. buy 2, pay 1
        b.add_product(p)
        self.assertEqual(b.total_gross(), 11.0)

if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestEx1))

    # TODO uncomment the following test if you proceed

    # tests for exercise 2
    # suite.addTest(unittest.makeSuite(TestEx2))

    # tests for exercise 3
    # suite.addTest(unittest.makeSuite(TestEx3))

    unittest.TextTestRunner(verbosity=3).run(suite)


